@@ -18,10 +18,13 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+from __future__ import print_function
 from collections import defaultdict
 import os
+import sys
 
 import ansiblelint.utils
+import codecs
 
 
 class AnsibleLintRule(object):
@@ -57,13 +60,16 @@ class AnsibleLintRule(object):
         yaml = ansiblelint.utils.parse_yaml_linenumbers(text)
         if yaml:
             for task in ansiblelint.utils.get_action_tasks(yaml, file):
-                if 'skip_ansible_lint' in task.get('tags', []):
+                # An empty `tags` block causes `None` to be returned if
+                # the `or []` is not present - `task.get('tags', [])`
+                # does not suffice.
+                if 'skip_ansible_lint' in (task.get('tags') or []):
                     continue
                 if 'action' in task:
                     result = self.matchtask(file, task)
                     if result:
                         message = None
-                        if isinstance(result, str):
+                        if isinstance(result, basestring):
                             message = result
                         taskstr = "Task/Handler: " + ansiblelint.utils.task_to_str(task)
                         matches.append(Match(task[ansiblelint.utils.LINE_NUMBER_KEY], taskstr,
@@ -77,9 +83,15 @@ class AnsibleLintRule(object):
             for play in yaml:
                 result = self.matchplay(file, play)
                 if result:
-                    (section, message) = result
-                    matches.append(Match(play[ansiblelint.utils.LINE_NUMBER_KEY], section,
-                                   file['path'], self, message))
+                    if isinstance(result, tuple):
+                        result = [result]
+
+                    if not isinstance(result, list):
+                        raise Exception("{} is not a list".format(result))
+
+                    for section, message in result:
+                        matches.append(Match(play[ansiblelint.utils.LINE_NUMBER_KEY],
+                                             section, file['path'], self, message))
         return matches
 
 
@@ -103,10 +115,18 @@ class RulesCollection(object):
     def run(self, playbookfile, tags=set(), skip_list=set()):
         text = ""
         matches = list()
-        with open(playbookfile['path'], 'Ur') as f:
-            text = f.read()
+
+        try:
+            with codecs.open(playbookfile['path'], mode='rb', encoding='utf-8') as f:
+                text = f.read()
+        except IOError, e:
+            print("WARNING: Couldn't open %s - %s" %
+                  (playbookfile['path'], e.strerror),
+                  file=sys.stderr)
+            return matches
+
         for rule in self.rules:
-            if not tags or not set(rule.tags).isdisjoint(tags):
+            if not tags or not set(rule.tags).union([rule.id]).isdisjoint(tags):
                 rule_definition = set(rule.tags)
                 rule_definition.add(rule.id)
                 if set(rule_definition).isdisjoint(skip_list):
@@ -137,7 +157,7 @@ class RulesCollection(object):
         return result
 
 
-class Match:
+class Match(object):
 
     def __init__(self, linenumber, line, filename, rule, message=None):
         self.linenumber = linenumber
@@ -147,21 +167,27 @@ class Match:
         self.message = message or rule.shortdesc
 
     def __repr__(self):
-        formatstr = "[{0}] ({1}) matched {2}:{3} {4}"
+        formatstr = u"[{0}] ({1}) matched {2}:{3} {4}"
         return formatstr.format(self.rule.id, self.message,
                                 self.filename, self.linenumber, self.line)
 
 
-class Runner:
+class Runner(object):
 
-    def __init__(self, rules, playbooks, tags, skip_list, exclude_paths):
+    def __init__(self, rules, playbooks, tags, skip_list, exclude_paths,
+                 verbosity=0):
         self.rules = rules
         self.playbooks = set()
         for pb in playbooks:
-            self.playbooks.add((pb, 'playbook'))
+            # assume role if directory
+            if os.path.isdir(pb):
+                self.playbooks.add((pb, 'role'))
+            else:
+                self.playbooks.add((pb, 'playbook'))
         self.tags = tags
         self.skip_list = skip_list
         self._update_exclude_paths(exclude_paths)
+        self.verbosity = verbosity
 
     def _update_exclude_paths(self, exclude_paths):
         if exclude_paths:
@@ -185,19 +211,23 @@ class Runner:
         for playbook in self.playbooks:
             if self.is_excluded(playbook[0]):
                 continue
+            if playbook[1] == 'role':
+                continue
             files.append({'path': playbook[0], 'type': playbook[1]})
         visited = set()
         while (visited != self.playbooks):
             for arg in self.playbooks - visited:
-                for file in ansiblelint.utils.find_children(arg):
-                    if self.is_excluded(file['path']):
+                for child in ansiblelint.utils.find_children(arg):
+                    if self.is_excluded(child['path']):
                         continue
-                    self.playbooks.add((file['path'], file['type']))
-                    files.append(file)
+                    self.playbooks.add((child['path'], child['type']))
+                    files.append(child)
                 visited.add(arg)
 
         matches = list()
         for file in files:
+            if self.verbosity > 0:
+                print("Examining %s of type %s" % (file['path'], file['type']))
             matches.extend(self.rules.run(file, tags=set(self.tags),
                            skip_list=set(self.skip_list)))
 
