@@ -19,7 +19,12 @@
 # THE SOFTWARE.
 
 import os
+
 from ansiblelint import AnsibleLintRule
+try:
+    from ansible.utils.boolean import boolean
+except ImportError:
+    from ansible.utils import boolean
 
 
 class CommandsInsteadOfModulesRule(AnsibleLintRule):
@@ -32,12 +37,13 @@ class CommandsInsteadOfModulesRule(AnsibleLintRule):
     _commands = ['command', 'shell']
     _modules = {'git': 'git', 'hg': 'hg', 'curl': 'get_url or uri', 'wget': 'get_url or uri',
                 'svn': 'subversion', 'service': 'service', 'mount': 'mount',
-                'rpm': 'yum', 'yum': 'yum', 'apt-get': 'apt-get',
+                'rpm': 'yum or rpm_key', 'yum': 'yum', 'apt-get': 'apt-get',
                 'unzip': 'unarchive', 'tar': 'unarchive'}
 
     def matchtask(self, file, task):
-        if task["action"]["module"] in self._commands:
+        if task["action"]["module"] in self._commands and task["action"]["module_arguments"]:
             executable = os.path.basename(task["action"]["module_arguments"][0])
-            if executable in self._modules:
+            if executable in self._modules and \
+                    boolean(task['action'].get('warn', True)):
                 message = "{0} used in place of {1} module"
                 return message.format(executable, self._modules[executable])
