@@ -18,19 +18,30 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-import os
 import glob
 import imp
+import os
+
 import ansible.constants as C
+from ansible.errors import AnsibleError
 from ansible.module_utils.splitter import split_args
 import yaml
 from yaml.composer import Composer
 from yaml.constructor import Constructor
 
 try:
-    from ansible.utils import parse_yaml_from_file, path_dwim
+    from ansible.utils import parse_yaml_from_file
+    from ansible.utils import path_dwim
+    from ansible.utils.template import template as ansible_template
+    from ansible.utils import module_finder
+    module_loader = module_finder
+    ANSIBLE_VERSION = 1
 except ImportError:
     from ansible.parsing.dataloader import DataLoader
+    from ansible.template import Templar
+    from ansible.parsing.mod_args import ModuleArgsParser
+    from ansible.plugins import module_loader
+    ANSIBLE_VERSION = 2
 
     def parse_yaml_from_file(filepath):
         dl = DataLoader()
@@ -41,17 +52,25 @@ except ImportError:
         dl.set_basedir(basedir)
         return dl.path_dwim(given)
 
+    def ansible_template(basedir, varname, templatevars, **kwargs):
+        dl = DataLoader()
+        dl.set_basedir(basedir)
+        templar = Templar(dl, variables=templatevars)
+        return templar.template(varname, **kwargs)
+
 LINE_NUMBER_KEY = '__line__'
 
 VALID_KEYS = [
-    'name', 'meta', 'action', 'when', 'async', 'poll', 'notify',
+    'name', 'action', 'when', 'async', 'poll', 'notify',
     'first_available_file', 'include', 'tags', 'register', 'ignore_errors',
     'delegate_to', 'local_action', 'transport', 'remote_user', 'sudo', 'sudo_user',
     'sudo_pass', 'when', 'connection', 'environment', 'args',
     'any_errors_fatal', 'changed_when', 'failed_when', 'always_run', 'delay', 'retries', 'until',
     'su', 'su_user', 'su_pass', 'no_log', 'run_once',
+    'become', 'become_user', 'become_method',
 ]
 
+
 def load_plugins(directory):
     result = []
     fh = None
@@ -71,7 +90,6 @@ def load_plugins(directory):
 
 
 def tokenize(line):
-    result = list()
     tokens = line.lstrip().split(" ")
     if tokens[0] == '-':
         tokens = tokens[1:]
@@ -102,10 +120,16 @@ def _playbook_items(pb_data):
 def find_children(playbook):
     if not os.path.exists(playbook[0]):
         return []
+    if playbook[1] == 'role':
+        playbook_ds = {'roles': [{'role': playbook[0]}]}
+    else:
+        try:
+            playbook_ds = parse_yaml_from_file(playbook[0])
+        except AnsibleError, e:
+            raise SystemExit(str(e))
     results = []
     basedir = os.path.dirname(playbook[0])
-    pb_data = parse_yaml_from_file(playbook[0])
-    items = _playbook_items(pb_data)
+    items = _playbook_items(playbook_ds)
     for item in items:
         for child in play_children(basedir, item, playbook[1]):
             if "$" in child['path'] or "{{" in child['path']:
@@ -123,6 +147,18 @@ def find_children(playbook):
     return results
 
 
+def template(basedir, value, vars, fail_on_undefined=False, **kwargs):
+    try:
+        value = ansible_template(os.path.abspath(basedir), value, vars,
+                                 **dict(kwargs, fail_on_undefined=fail_on_undefined))
+        # Hack to skip the following exception when using to_json filter on a variable.
+        # I guess the filter doesn't like empty vars...
+    except (AnsibleError, ValueError):
+        # templating failed, so just keep value as is.
+        pass
+    return value
+
+
 def play_children(basedir, item, parent_type):
     delegate_map = {
         'tasks': _taskshandlers_children,
@@ -136,12 +172,23 @@ def play_children(basedir, item, parent_type):
     (k, v) = item
     if k in delegate_map:
         if v:
+            v = template(
+                    os.path.abspath(basedir),
+                    v,
+                    dict(playbook_dir=os.path.abspath(basedir)),
+                    fail_on_undefined=False)
             return delegate_map[k](basedir, k, v, parent_type)
     return []
 
 
 def _include_children(basedir, k, v, parent_type):
-    return [{'path': path_dwim(basedir, v), 'type': parent_type}]
+    # handle include: filename.yml tags=blah
+    (command, args, kwargs) = tokenize("{0}: {1}".format(k, v))
+
+    result = path_dwim(basedir, args[0])
+    if not os.path.exists(result) and not basedir.endswith('tasks'):
+        result = path_dwim(os.path.join(basedir, '..', 'tasks'), v)
+    return [{'path': result, 'type': parent_type}]
 
 
 def _taskshandlers_children(basedir, k, v, parent_type):
@@ -154,7 +201,11 @@ def _roles_children(basedir, k, v, parent_type):
     results = []
     for role in v:
         if isinstance(role, dict):
-            results.extend(_look_for_role_files(basedir, role['role']))
+            if 'role' in role:
+                if 'tags' not in role or 'skip_ansible_lint' not in role['tags']:
+                    results.extend(_look_for_role_files(basedir, role['role']))
+            else:
+                raise SystemExit('role dict {0} does not contain a "role" key'.format(role))
         else:
             results.extend(_look_for_role_files(basedir, role))
     return results
@@ -171,12 +222,13 @@ def _rolepath(basedir, role):
         path_dwim(
             basedir, os.path.join('..', '..', '..', 'roles', role)
         ),
-        path_dwim(basedir,
-                                os.path.join('..', '..', role))
+        path_dwim(basedir, os.path.join('..', '..', role))
     ]
 
     if C.DEFAULT_ROLES_PATH:
-        search_locations = C.DEFAULT_ROLES_PATH.split(os.pathsep)
+        search_locations = C.DEFAULT_ROLES_PATH
+        if isinstance(search_locations, basestring):
+            search_locations = search_locations.split(os.pathsep)
         for loc in search_locations:
             loc = os.path.expanduser(loc)
             possible_paths.append(path_dwim(loc, role))
@@ -186,6 +238,11 @@ def _rolepath(basedir, role):
             role_path = path_option
             break
 
+    if role_path:
+        role_modules = os.path.join(role_path, 'library')
+        if os.path.exists(role_modules):
+            module_loader.add_directory(role_modules)
+
     return role_path
 
 
@@ -219,10 +276,38 @@ def _kv_to_dict(v):
     return (dict(module=command, module_arguments=args, **kwargs))
 
 
-def normalize_task(task):
-    ''' ensures that all tasks have an action key
-        and that string values are converted to python objects '''
+def normalize_task_v2(task):
+    '''Ensures tasks have an action key and strings are converted to python objects'''
 
+    result = dict()
+    mod_arg_parser = ModuleArgsParser(task)
+    action, arguments, result['delegate_to'] = mod_arg_parser.parse()
+
+    # denormalize shell -> command conversion
+    if '_use_shell' in arguments:
+        action = 'shell'
+        del(arguments['_use_shell'])
+
+    for (k, v) in list(task.items()):
+        if k in ('action', 'local_action', 'args', 'delegate_to') or k == action:
+            # we don't want to re-assign these values, which were
+            # determined by the ModuleArgsParser() above
+            continue
+        else:
+            result[k] = v
+
+    result['action'] = dict(module=action)
+
+    if '_raw_params' in arguments:
+        result['action']['module_arguments'] = arguments['_raw_params'].split()
+        del(arguments['_raw_params'])
+    else:
+        result['action']['module_arguments'] = list()
+    result['action'].update(arguments)
+    return result
+
+
+def normalize_task_v1(task):
     result = dict()
     for (k, v) in task.items():
         if k in VALID_KEYS or k.startswith('with_'):
@@ -248,35 +333,66 @@ def normalize_task(task):
                         continue
 
                     else:
-                        # Should not get here!
-                        print "Was not expecting value %s of type %s for key %s" % (str(v), type(v), k)
-                        print "Task: %s" % str(task)
-                        exit(1)
+                        # Tasks that include playbooks (rather than task files)
+                        # can get here
+                        # https://github.com/willthames/ansible-lint/issues/138
+                        raise RuntimeError("Was not expecting value %s of type %s for key %s\n"
+                                           "Task: %s. Check the syntax of your playbook using "
+                                           "ansible-playbook --syntax-check" %
+                                           (str(v), type(v), k, str(task)))
             v['module_arguments'] = v.get('module_arguments', list())
             result['action'] = v
+    if 'args' in result:
+        result['action'].update(result.get('args'))
+        del(result['args'])
     return result
 
 
+def normalize_task(task):
+    if ANSIBLE_VERSION < 2:
+        return normalize_task_v1(task)
+    else:
+        return normalize_task_v2(task)
+
+
 def task_to_str(task):
     name = task.get("name")
     if name:
         return name
     action = task.get("action")
-    args = " ".join(["k=v" for (k, v) in action.items() if k != "module_arguments"] +
-                    action.get("module_arguments"))
+    args = " " .join(["{0}={1}".format(k, v) for (k, v) in action.items()
+                     if k not in ["module", "module_arguments"]] +
+                     action.get("module_arguments"))
     return "{0} {1}".format(action["module"], args)
 
 
+def extract_from_list(blocks, candidates):
+    results = list()
+    for block in blocks:
+        for candidate in candidates:
+            if candidate in block:
+                if isinstance(block[candidate], list):
+                    results.extend(block[candidate])
+                elif block[candidate] is not None:
+                    raise RuntimeError(
+                        "Key '%s' defined, but bad value: '%s'" %
+                        (candidate, str(block[candidate])))
+    return results
+
+
 def get_action_tasks(yaml, file):
     tasks = list()
     if file['type'] in ['tasks', 'handlers']:
         tasks = yaml
     else:
-        for block in yaml:
-            for section in ['tasks', 'handlers', 'pre_tasks', 'post_tasks']:
-                if section in block:
-                    block_tasks = block.get(section) or []
-                    tasks.extend(block_tasks)
+        tasks.extend(extract_from_list(yaml, ['tasks', 'handlers', 'pre_tasks', 'post_tasks']))
+
+    # Add sub-elements of block/rescue/always to tasks list
+    tasks.extend(extract_from_list(tasks, ['block', 'rescue', 'always']))
+    # Remove block/rescue/always elements from tasks list
+    block_rescue_always = ('block', 'rescue', 'always')
+    tasks[:] = [task for task in tasks if all(k not in task for k in block_rescue_always)]
+
     return [normalize_task(task) for task in tasks
             if 'include' not in task.keys()]
 
@@ -284,7 +400,8 @@ def get_action_tasks(yaml, file):
 def parse_yaml_linenumbers(data):
     """Parses yaml as ansible.utils.parse_yaml but with linenumbers.
 
-    The line numbers are stored in each node's LINE_NUMBER_KEY key"""
+    The line numbers are stored in each node's LINE_NUMBER_KEY key.
+    """
     loader = yaml.Loader(data)
 
     def compose_node(parent, index):
